// @(#)root/reflex:$Name:  $:$Id: FunctionMemberTemplateInstance.h,v 1.1 2005/06/23 10:23:22 brun Exp $
// Author: Stefan Roiser 2004

// Copyright CERN, CH-1211 Geneva 23, 2004-2005, All rights reserved.
//
// Permission to use, copy, modify, and distribute this software for any
// purpose is hereby granted without fee, provided that this copyright and
// permissions notice appear in all copies and derivatives.
//
// This software is provided "as is" without express or implied warranty.

#ifndef ROOT_Reflex_FunctionMemberTemplateInstance
#define ROOT_Reflex_FunctionMemberTemplateInstance

// Include files
#include "FunctionMember.h"
#include "TemplateInstance.h"

namespace ROOT {
  namespace Reflex {

    // forward declarations
    class Type;

    /**
     * @class FunctionMemberTemplateInstance FunctionMemberTemplateInstance.h Reflex/FunctionMemberTemplateInstance.h
     * @author Stefan Roiser
     * @date 13/1/2004
     * @ingroup Ref
     */
    class FunctionMemberTemplateInstance : public FunctionMember, public TemplateInstance {
    
    public:

      /** default constructor */
      FunctionMemberTemplateInstance( const char * Name,
                                      const Type & TypeNth,
                                      StubFunction stubFP,
                                      void * stubCtx = 0,
                                      const char * params = 0, 
                                      unsigned int modifiers = 0,
                                      const Scope & ScopeNth = Scope());


      /** destructor */
      virtual ~FunctionMemberTemplateInstance();


      /**
       * Name returns the fully qualified Name of the
       * templated function
       * @param  typedefexp expand typedefs or not
       * @return fully qualified Name of templated function
       */
      std::string Name( unsigned int mod = 0 ) const;


      /**
       * TemplateArgumentNth will return a pointer to the nth template argument
       * @param  nth nth template argument
       * @return pointer to nth template argument
       */
      Type TemplateArgumentNth( size_t nth ) const;


      /**
       * templateArgCount will return the number of template arguments
       * @return number of template arguments
       */
      size_t TemplateArgumentCount() const;


      /**
       * TemplateFamily returns the corresponding MemberTemplate if any
       * @return corresponding MemberTemplate
       */
      MemberTemplate TemplateFamily() const;

    private:

      /** 
       * The template TypeNth (family)
       * @label template family
       * @link aggregationByValue
       * @clientCardinality 1
       * @supplierCardinality 0..1
       */
      MemberTemplate fTemplateFamily;      

    }; // class FunctionMemberTemplateInstance
  } // namespace Reflex
} // namespace ROOT

//-------------------------------------------------------------------------------
inline ROOT::Reflex::FunctionMemberTemplateInstance::~FunctionMemberTemplateInstance() {}
//-------------------------------------------------------------------------------


//-------------------------------------------------------------------------------
inline size_t ROOT::Reflex::FunctionMemberTemplateInstance::TemplateArgumentCount() const {
//-------------------------------------------------------------------------------
  return TemplateInstance::TemplateArgumentCount();
}


//-------------------------------------------------------------------------------
inline ROOT::Reflex::MemberTemplate ROOT::Reflex::FunctionMemberTemplateInstance::TemplateFamily() const {
//-------------------------------------------------------------------------------
  return fTemplateFamily;
}

#endif // ROOT_Reflex_FunctionMemberTemplateInstance
