// @(#)root/reflex:$Name:  $:$Id: Object.cxx,v 1.1 2005/06/23 10:21:06 brun Exp $
// Author: Stefan Roiser 2004

// Copyright CERN, CH-1211 Geneva 23, 2004-2005, All rights reserved.
//
// Permission to use, copy, modify, and distribute this software for any
// purpose is hereby granted without fee, provided that this copyright and
// permissions notice appear in all copies and derivatives.
//
// This software is provided "as is" without express or implied warranty.

#include "Reflex/Object.h"

#include "Class.h"
#include "DataMember.h"
#include "FunctionMember.h"


//-------------------------------------------------------------------------------
ROOT::Reflex::Object 
ROOT::Reflex::Object::Get( const std::string & dm ) const {
//-------------------------------------------------------------------------------
  Member m = TypeGet().MemberNth( dm );
  if ( m ) return m.Get( * this );
  else throw RuntimeError("No such MemberNth " + dm );
  return Object();
}


//-------------------------------------------------------------------------------
template < class T > 
T ROOT::Reflex::Object::GetT( const std::string & dm ) const {
//-------------------------------------------------------------------------------
  Member m = TypeGet().MemberNth( dm );
  if ( m ) return object_cast< T > ( m.Get( * this ));
  else throw RuntimeError("No such MemberNth " + dm );
  return object_cast < T > ( Object() );
}


//-------------------------------------------------------------------------------
//void ROOT::Reflex::Object::Set( const std::string & dm,
//                                const Object & value ) const {
//-------------------------------------------------------------------------------
//  Member m = TypeGet().MemberNth( dm );
//  if ( m ) m.Set( * this, value );
//  else throw RuntimeError("No such MemberNth " + dm );
//}


//-------------------------------------------------------------------------------
void ROOT::Reflex::Object::Set( const std::string & dm,
                                const void * value ) const {
//-------------------------------------------------------------------------------
  Member m = TypeGet().MemberNth( dm );
  if ( m ) m.Set( * this, value );
  else throw RuntimeError("No such MemberNth " + dm );
}


/*/-------------------------------------------------------------------------------
ROOT::Reflex::Object
ROOT::Reflex::Object::Invoke( const std::string & fm,
                              std::vector< Object > args ) const {
//-------------------------------------------------------------------------------
  Member m = TypeGet().MemberNth( fm );
  if ( m ) {
    if ( args.size() ) return m.Invoke( * this, args );
    else               return m.Invoke( * this );
  }
  else throw RuntimeError("No such MemberNth " + fm );
  return Object();
}
*/


//-------------------------------------------------------------------------------
ROOT::Reflex::Object
ROOT::Reflex::Object::Invoke( const std::string & fm,
                              std::vector < void * > args ) const {
//-------------------------------------------------------------------------------
  Member m = TypeGet().MemberNth( fm );
  if ( m ) {
    if ( args.size() ) return m.Invoke( * this, args );
    else               return m.Invoke( * this );
  }
  else throw RuntimeError("No such MemberNth " + fm );
  return Object();
}


//-------------------------------------------------------------------------------
template < class T0 > ROOT::Reflex::Object
ROOT::Reflex::Object::Invoke( const std::string & fm,
                              const T0 & p0 ) const {
//-------------------------------------------------------------------------------
  Member m = TypeGet().MemberNth( fm );
  if ( m ) {
    std::vector< Object > argList;
    argList.push_back( Object( Type::ByTypeInfo( typeid(T0) ), & p0 ));
    return m.Invoke( * this, argList );
  }
  else throw RuntimeError("No such MemberNth " + fm );
  return Object();
}


//-------------------------------------------------------------------------------
template < class T0, class T1 > ROOT::Reflex::Object
ROOT::Reflex::Object::Invoke( const std::string & fm,
                              const T0 & p0,
                              const T1 & p1 ) const {
//-------------------------------------------------------------------------------
  Member m = TypeGet().MemberNth( fm );
  if ( m ) {
    std::vector< Object > argList;
    argList.push_back( Object( Type::ByTypeInfo( typeid(T0) ), & p0 ));
    argList.push_back( Object( Type::ByTypeInfo( typeid(T1) ), & p1 ));
    return m.Invoke( * this, argList );
  }
  else throw RuntimeError("No such MemberNth " + fm );
  return Object();
}


//-------------------------------------------------------------------------------
ROOT::Reflex::Object ROOT::Reflex::Object::Field( const std::string & data ) const {
//-------------------------------------------------------------------------------
  Type t = TypeGet();
  if ( ! t.IsClass() ) throw RuntimeError("Object is not a composite");
  for ( size_t i = 0; i < t.DataMemberCount(); ++i ) {
    Member dm = t.DataMemberNth( i );
    if ( dm.Name() == data ) {
      return Object(dm.TypeGet(), (void*)((char*)AddressGet() + dm.Offset()));
    }
  }
  throw RuntimeError("Data MemberNth not found in class");
}
 
